@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
@@ -28,6 +25,7 @@
 
 guac_pool* guac_pool_alloc(int size) {
 
+    pthread_mutexattr_t lock_attributes;
     guac_pool* pool = malloc(sizeof(guac_pool));
 
     /* If unable to allocate, just return NULL. */
@@ -41,6 +39,11 @@ guac_pool* guac_pool_alloc(int size) {
     pool->__head = NULL;
     pool->__tail = NULL;
 
+    /* Init lock */
+    pthread_mutexattr_init(&lock_attributes);
+    pthread_mutexattr_setpshared(&lock_attributes, PTHREAD_PROCESS_SHARED);
+    pthread_mutex_init(&(pool->__lock), &lock_attributes);
+
     return pool;
 
 }
@@ -57,6 +60,9 @@ void guac_pool_free(guac_pool* pool) {
         free(old);
     }
 
+    /* Destroy lock */
+    pthread_mutex_destroy(&(pool->__lock));
+
     /* Free pool */
     free(pool);
 
@@ -66,11 +72,17 @@ int guac_pool_next_int(guac_pool* pool) {
 
     int value;
 
+    /* Acquire exclusive access */
+    pthread_mutex_lock(&(pool->__lock));
+
     pool->active++;
 
     /* If more integers are needed, return a new one. */
-    if (pool->__head == NULL || pool->__next_value < pool->min_size)
-        return pool->__next_value++;
+    if (pool->__head == NULL || pool->__next_value < pool->min_size) {
+        value = pool->__next_value++;
+        pthread_mutex_unlock(&(pool->__lock));
+        return value;
+    }
 
     /* Otherwise, remove first integer. */
     value = pool->__head->value;
@@ -90,6 +102,7 @@ int guac_pool_next_int(guac_pool* pool) {
     }
 
     /* Return retrieved value. */
+    pthread_mutex_unlock(&(pool->__lock));
     return value;
 }
 
@@ -100,6 +113,9 @@ void guac_pool_free_int(guac_pool* pool, int value) {
     pool_int->value = value;
     pool_int->__next = NULL;
 
+    /* Acquire exclusive access */
+    pthread_mutex_lock(&(pool->__lock));
+
     pool->active--;
 
     /* If pool empty, store as sole entry. */
@@ -112,5 +128,8 @@ void guac_pool_free_int(guac_pool* pool, int value) {
         pool->__tail = pool_int;
     }
 
+    /* Value has been freed */
+    pthread_mutex_unlock(&(pool->__lock));
+
 }
 
