@@ -1,28 +1,25 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
-#include "client.h"
+#include "rdp.h"
 #include "rdp_fs.h"
 #include "rdp_settings.h"
 #include "rdp_stream.h"
@@ -91,19 +88,19 @@ void guac_rdpdr_process_connect(rdpSvcPlugin* plugin) {
     plugin->channel_entry_points.pExtendedData = NULL;
 
     /* Get data from client */
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     /* Init plugin */
     rdpdr->client = client;
     rdpdr->devices_registered = 0;
 
     /* Register printer if enabled */
-    if (client_data->settings.printing_enabled)
-        guac_rdpdr_register_printer(rdpdr);
+    if (rdp_client->settings->printing_enabled)
+        guac_rdpdr_register_printer(rdpdr, rdp_client->settings->printer_name);
 
     /* Register drive if enabled */
-    if (client_data->settings.drive_enabled)
-        guac_rdpdr_register_fs(rdpdr);
+    if (rdp_client->settings->drive_enabled)
+        guac_rdpdr_register_fs(rdpdr, rdp_client->settings->drive_name);
 
     /* Log that printing, etc. has been loaded */
     guac_client_log(client, GUAC_LOG_INFO, "guacdr connected.");
@@ -222,12 +219,38 @@ wStream* guac_rdpdr_new_io_completion(guac_rdpdr_device* device,
 
 }
 
-void guac_rdpdr_start_download(guac_rdpdr_device* device, const char* path) {
+/**
+ * Callback invoked on the current connection owner (if any) when a file
+ * download is being initiated using the magic "Download" folder.
+ *
+ * @param owner
+ *     The guac_user that is the owner of the connection, or NULL if the
+ *     connection owner has left.
+ *
+ * @param data
+ *     The full absolute path to the file that should be downloaded.
+ *
+ * @return
+ *     The stream allocated for the file download, or NULL if the download has
+ *     failed to start.
+ */
+static void* guac_rdpdr_download_to_owner(guac_user* owner, void* data) {
 
-    /* Get client and stream */
-    guac_client* client = device->rdpdr->client;
+    /* Do not bother attempting the download if the owner has left */
+    if (owner == NULL)
+        return NULL;
 
-    int file_id = guac_rdp_fs_open((guac_rdp_fs*) device->data, path,
+    guac_client* client = owner->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_rdp_fs* filesystem = rdp_client->filesystem;
+
+    /* Ignore download if filesystem has been unloaded */
+    if (filesystem == NULL)
+        return NULL;
+
+    /* Attempt to open requested file */
+    char* path = (char*) data;
+    int file_id = guac_rdp_fs_open(filesystem, path,
             ACCESS_FILE_READ_DATA, 0, DISP_FILE_OPEN, 0);
 
     /* If file opened successfully, start stream */
@@ -240,7 +263,7 @@ void guac_rdpdr_start_download(guac_rdpdr_device* device, const char* path) {
         char c;
 
         /* Associate stream with transfer status */
-        guac_stream* stream = guac_client_alloc_stream(client);
+        guac_stream* stream = guac_user_alloc_stream(owner);
         stream->data = rdp_stream = malloc(sizeof(guac_rdp_stream));
         stream->ack_handler = guac_rdp_download_ack_handler;
         rdp_stream->type = GUAC_RDP_DOWNLOAD_STREAM;
@@ -260,17 +283,31 @@ void guac_rdpdr_start_download(guac_rdpdr_device* device, const char* path) {
 
         } while (c != '\0');
 
-        guac_client_log(device->rdpdr->client, GUAC_LOG_DEBUG,
-                "%s: Initiating download of \"%s\"", __func__, path);
+        guac_user_log(owner, GUAC_LOG_DEBUG, "%s: Initiating download "
+                "of \"%s\"", __func__, path);
 
         /* Begin stream */
-        guac_protocol_send_file(client->socket, stream,
+        guac_protocol_send_file(owner->socket, stream,
                 "application/octet-stream", basename);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(owner->socket);
+
+        /* Download started successfully */
+        return stream;
 
     }
-    else
-        guac_client_log(client, GUAC_LOG_ERROR, "Unable to download \"%s\"", path);
+
+    /* Download failed */
+    guac_user_log(owner, GUAC_LOG_ERROR, "Unable to download \"%s\"", path);
+    return NULL;
+
+}
+
+void guac_rdpdr_start_download(guac_rdpdr_device* device, char* path) {
+
+    guac_client* client = device->rdpdr->client;
+
+    /* Initiate download to the owner of the connection */
+    guac_client_for_owner(client, guac_rdpdr_download_to_owner, path);
 
 }
 
