@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 
@@ -77,15 +74,30 @@ struct guac_rdpdr_device {
     int device_id;
 
     /**
-     * An arbitrary device name, used for logging purposes only.
+     * Device name, used for logging and for passthrough to the
+     * server.
      */
     const char* device_name;
 
     /**
-     * Handler which will be called when the RDPDR plugin is forming the client
-     * device announce list.
+     * The type of RDPDR device that this represents.
      */
-    guac_rdpdr_device_announce_handler* announce_handler;
+    uint32_t device_type;
+
+    /**
+     * The DOS name of the device.  Max 8 bytes, including terminator.
+     */
+    const char *dos_name;
+    
+    /**
+     * The stream that stores the RDPDR device announcement for this device.
+     */
+    wStream* device_announce;
+    
+    /**
+     * The length of the device_announce wStream.
+     */
+    int device_announce_len;
 
     /**
      * Handler which should be called for every I/O request received.
@@ -93,7 +105,7 @@ struct guac_rdpdr_device {
     guac_rdpdr_device_iorequest_handler* iorequest_handler;
 
     /**
-     * Handlel which should be called when the device is being free'd.
+     * Handler which should be called when the device is being freed.
      */
     guac_rdpdr_device_free_handler* free_handler;
 
@@ -157,7 +169,7 @@ void guac_rdpdr_process_terminate(rdpSvcPlugin* plugin);
 void guac_rdpdr_process_event(rdpSvcPlugin* plugin, wMessage* event);
 
 /**
- * Creates a new stream which contains the ommon DR_DEVICE_IOCOMPLETION header
+ * Creates a new stream which contains the common DR_DEVICE_IOCOMPLETION header
  * used for virtually all responses.
  */
 wStream* guac_rdpdr_new_io_completion(guac_rdpdr_device* device,
@@ -166,7 +178,7 @@ wStream* guac_rdpdr_new_io_completion(guac_rdpdr_device* device,
 /**
  * Begins streaming the given file to the user via a Guacamole file stream.
  */
-void guac_rdpdr_start_download(guac_rdpdr_device* device, const char* path);
+void guac_rdpdr_start_download(guac_rdpdr_device* device, char* path);
 
 #endif
 
