@@ -1,29 +1,28 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
 #include "client.h"
-#include "guac_surface.h"
+#include "common/display.h"
+#include "common/surface.h"
+#include "rdp.h"
 #include "rdp_bitmap.h"
 #include "rdp_settings.h"
 
@@ -46,12 +45,11 @@
 void guac_rdp_cache_bitmap(rdpContext* context, rdpBitmap* bitmap) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_socket* socket = client->socket; 
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
-    /* Allocate surface */
-    guac_layer* buffer = guac_client_alloc_buffer(client);
-    guac_common_surface* surface = guac_common_surface_alloc(client, socket,
-            buffer, bitmap->width, bitmap->height);
+    /* Allocate buffer */
+    guac_common_display_layer* buffer = guac_common_display_alloc_buffer(
+            rdp_client->display, bitmap->width, bitmap->height);
 
     /* Cache image data if present */
     if (bitmap->data != NULL) {
@@ -62,7 +60,7 @@ void guac_rdp_cache_bitmap(rdpContext* context, rdpBitmap* bitmap) {
             bitmap->width, bitmap->height, 4*bitmap->width);
 
         /* Send surface to buffer */
-        guac_common_surface_draw(surface, 0, 0, image);
+        guac_common_surface_draw(buffer->surface, 0, 0, image);
 
         /* Free surface */
         cairo_surface_destroy(image);
@@ -70,8 +68,7 @@ void guac_rdp_cache_bitmap(rdpContext* context, rdpBitmap* bitmap) {
     }
 
     /* Store buffer reference in bitmap */
-    ((guac_rdp_bitmap*) bitmap)->buffer = buffer;
-    ((guac_rdp_bitmap*) bitmap)->surface = surface;
+    ((guac_rdp_bitmap*) bitmap)->layer = buffer;
 
 }
 
@@ -101,8 +98,7 @@ void guac_rdp_bitmap_new(rdpContext* context, rdpBitmap* bitmap) {
     }
 
     /* No corresponding surface yet - caching is deferred. */
-    ((guac_rdp_bitmap*) bitmap)->buffer = NULL;
-    ((guac_rdp_bitmap*) bitmap)->surface = NULL;
+    ((guac_rdp_bitmap*) bitmap)->layer = NULL;
 
     /* Start at zero usage */
     ((guac_rdp_bitmap*) bitmap)->used = 0;
@@ -112,21 +108,22 @@ void guac_rdp_bitmap_new(rdpContext* context, rdpBitmap* bitmap) {
 void guac_rdp_bitmap_paint(rdpContext* context, rdpBitmap* bitmap) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
-    guac_common_surface* surface = ((guac_rdp_bitmap*) bitmap)->surface;
+    guac_common_display_layer* buffer = ((guac_rdp_bitmap*) bitmap)->layer;
 
     int width = bitmap->right - bitmap->left + 1;
     int height = bitmap->bottom - bitmap->top + 1;
 
     /* If not cached, cache if necessary */
-    if (surface == NULL && ((guac_rdp_bitmap*) bitmap)->used >= 1)
+    if (buffer == NULL && ((guac_rdp_bitmap*) bitmap)->used >= 1)
         guac_rdp_cache_bitmap(context, bitmap);
 
     /* If cached, retrieve from cache */
-    if (surface != NULL)
-        guac_common_surface_copy(surface, 0, 0, width, height,
-                                 client_data->default_surface, bitmap->left, bitmap->top);
+    if (buffer != NULL)
+        guac_common_surface_copy(buffer->surface, 0, 0, width, height,
+                rdp_client->display->default_surface,
+                bitmap->left, bitmap->top);
 
     /* Otherwise, draw with stored image data */
     else if (bitmap->data != NULL) {
@@ -137,7 +134,8 @@ void guac_rdp_bitmap_paint(rdpContext* context, rdpBitmap* bitmap) {
             width, height, 4*bitmap->width);
 
         /* Draw image on default surface */
-        guac_common_surface_draw(client_data->default_surface, bitmap->left, bitmap->top, image);
+        guac_common_surface_draw(rdp_client->display->default_surface,
+                bitmap->left, bitmap->top, image);
 
         /* Free surface */
         cairo_surface_destroy(image);
@@ -152,26 +150,22 @@ void guac_rdp_bitmap_paint(rdpContext* context, rdpBitmap* bitmap) {
 void guac_rdp_bitmap_free(rdpContext* context, rdpBitmap* bitmap) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_layer* buffer = ((guac_rdp_bitmap*) bitmap)->buffer;
-    guac_common_surface* surface = ((guac_rdp_bitmap*) bitmap)->surface;
-
-    /* If cached, free surface */
-    if (surface != NULL)
-        guac_common_surface_free(surface);
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_common_display_layer* buffer = ((guac_rdp_bitmap*) bitmap)->layer;
 
     /* If cached, free buffer */
     if (buffer != NULL)
-        guac_client_free_buffer(client, buffer);
+        guac_common_display_free_buffer(rdp_client->display, buffer);
 
 }
 
 void guac_rdp_bitmap_setsurface(rdpContext* context, rdpBitmap* bitmap, BOOL primary) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     if (primary)
-        client_data->current_surface = client_data->default_surface;
+        rdp_client->current_surface = rdp_client->display->default_surface;
 
     else {
 
@@ -182,10 +176,11 @@ void guac_rdp_bitmap_setsurface(rdpContext* context, rdpBitmap* bitmap, BOOL pri
         }
 
         /* If not available as a surface, make available. */
-        if (((guac_rdp_bitmap*) bitmap)->surface == NULL)
+        if (((guac_rdp_bitmap*) bitmap)->layer == NULL)
             guac_rdp_cache_bitmap(context, bitmap);
 
-        client_data->current_surface = ((guac_rdp_bitmap*) bitmap)->surface;
+        rdp_client->current_surface =
+            ((guac_rdp_bitmap*) bitmap)->layer->surface;
 
     }
 
