@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
@@ -38,8 +35,11 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <guacamole/client.h>
 #include <guacamole/object.h>
 #include <guacamole/pool.h>
+#include <guacamole/socket.h>
+#include <guacamole/user.h>
 
 guac_rdp_fs* guac_rdp_fs_alloc(guac_client* client, const char* drive_path,
         int create_drive_path) {
@@ -61,10 +61,6 @@ guac_rdp_fs* guac_rdp_fs_alloc(guac_client* client, const char* drive_path,
     guac_rdp_fs* fs = malloc(sizeof(guac_rdp_fs));
 
     fs->client = client;
-    fs->object = guac_client_alloc_object(client);
-    fs->object->get_handler = guac_rdp_download_get_handler;
-    fs->object->put_handler = guac_rdp_upload_put_handler;
-
     fs->drive_path = strdup(drive_path);
     fs->file_id_pool = guac_pool_alloc(0);
     fs->open_files = 0;
@@ -74,16 +70,56 @@ guac_rdp_fs* guac_rdp_fs_alloc(guac_client* client, const char* drive_path,
 }
 
 void guac_rdp_fs_free(guac_rdp_fs* fs) {
-    guac_client_free_object(fs->client, fs->object);
     guac_pool_free(fs->file_id_pool);
     free(fs->drive_path);
     free(fs);
 }
 
+guac_object* guac_rdp_fs_alloc_object(guac_rdp_fs* fs, guac_user* user) {
+
+    /* Init filesystem */
+    guac_object* fs_object = guac_user_alloc_object(user);
+    fs_object->get_handler = guac_rdp_download_get_handler;
+    fs_object->put_handler = guac_rdp_upload_put_handler;
+    fs_object->data = fs;
+
+    /* Send filesystem to user */
+    guac_protocol_send_filesystem(user->socket, fs_object, "Shared Drive");
+    guac_socket_flush(user->socket);
+
+    return fs_object;
+
+}
+
+void* guac_rdp_fs_expose(guac_user* user, void* data) {
+
+    guac_rdp_fs* fs = (guac_rdp_fs*) data;
+
+    /* No need to expose if there is no filesystem or the user has left */
+    if (user == NULL || fs == NULL)
+        return NULL;
+
+    /* Allocate and expose filesystem object for user */
+    return guac_rdp_fs_alloc_object(fs, user);
+
+}
+
 /**
- * Translates an absolute Windows virtual_path to an absolute virtual_path
- * which is within the "drive virtual_path" specified in the connection
- * settings.
+ * Translates an absolute Windows path to an absolute path which is within the
+ * "drive path" specified in the connection settings. No checking is performed
+ * on the path provided, which is assumed to have already been normalized and
+ * validated as absolute.
+ *
+ * @param fs
+ *     The filesystem containing the file whose path is being translated.
+ *
+ * @param virtual_path
+ *     The absolute path to the file on the simulated filesystem, relative to
+ *     the simulated filesystem root.
+ *
+ * @param real_path
+ *     The buffer in which to store the absolute path to the real file on the
+ *     local filesystem.
  */
 static void __guac_rdp_fs_translate_path(guac_rdp_fs* fs,
         const char* virtual_path, char* real_path) {
@@ -559,16 +595,12 @@ const char* guac_rdp_fs_read_dir(guac_rdp_fs* fs, int file_id) {
             return NULL;
     }
 
-    /* Read next entry, stop if error */
-    if (readdir_r(file->dir, &(file->__dirent), &result))
-        return NULL;
-
-    /* If no more entries, return NULL */
-    if (result == NULL)
+    /* Read next entry, stop if error or no more entries */
+    if ((result = readdir(file->dir)) == NULL)
         return NULL;
 
     /* Return filename */
-    return file->__dirent.d_name;
+    return result->d_name;
 
 }
 
@@ -590,10 +622,10 @@ int guac_rdp_fs_normalize_path(const char* path, char* abs_path) {
     path++;
 
     /* Copy path into component data for parsing */
-    strncpy(path_component_data, path, GUAC_RDP_FS_MAX_PATH-1);
+    strncpy(path_component_data, path, sizeof(path_component_data) - 1);
 
     /* Find path components within path */
-    for (i=0; i<GUAC_RDP_FS_MAX_PATH; i++) {
+    for (i = 0; i < sizeof(path_component_data) - 1; i++) {
 
         /* If current character is a path separator, parse as component */
         char c = path_component_data[i];
@@ -622,6 +654,10 @@ int guac_rdp_fs_normalize_path(const char* path, char* abs_path) {
 
         } /* end if separator */
 
+        /* We do not currently support named streams */
+        else if (c == ':')
+            return 1;
+
     } /* end for each character */
 
     /* If no components, the path is simply root */
@@ -701,7 +737,7 @@ int guac_rdp_fs_get_info(guac_rdp_fs* fs, guac_rdp_fs_info* info) {
     /* Read FS information */
     struct statvfs fs_stat;
     if (statvfs(fs->drive_path, &fs_stat))
-        return guac_rdp_fs_get_status(errno);
+        return guac_rdp_fs_get_errorcode(errno);
 
     /* Assign to structure */
     info->blocks_available = fs_stat.f_bfree;
