@@ -1,29 +1,27 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
 #include "client.h"
-#include "guac_surface.h"
+#include "common/surface.h"
+#include "rdp.h"
 #include "rdp_bitmap.h"
 #include "rdp_color.h"
 #include "rdp_settings.h"
@@ -102,7 +100,7 @@ guac_transfer_function guac_rdp_rop3_transfer_function(guac_client* client,
 void guac_rdp_gdi_dstblt(rdpContext* context, DSTBLT_ORDER* dstblt) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_common_surface* current_surface = ((rdp_guac_client_data*) client->data)->current_surface;
+    guac_common_surface* current_surface = ((guac_rdp_client*) client->data)->current_surface;
 
     int x = dstblt->nLeftRect;
     int y = dstblt->nTopRect;
@@ -115,7 +113,8 @@ void guac_rdp_gdi_dstblt(rdpContext* context, DSTBLT_ORDER* dstblt) {
         case 0:
 
             /* Send black rectangle */
-            guac_common_surface_rect(current_surface, x, y, w, h, 0, 0, 0);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0x00, 0x00, 0x00, 0xFF);
             break;
 
         /* DSTINVERT */
@@ -130,7 +129,8 @@ void guac_rdp_gdi_dstblt(rdpContext* context, DSTBLT_ORDER* dstblt) {
 
         /* Whiteness */
         case 0xFF:
-            guac_common_surface_rect(current_surface, x, y, w, h, 0xFF, 0xFF, 0xFF);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0xFF, 0xFF, 0xFF, 0xFF);
             break;
 
         /* Unsupported ROP3 */
@@ -158,7 +158,7 @@ void guac_rdp_gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt) {
     /* Get client and current layer */
     guac_client* client = ((rdp_freerdp_context*) context)->client;
     guac_common_surface* current_surface =
-        ((rdp_guac_client_data*) client->data)->current_surface;
+        ((guac_rdp_client*) client->data)->current_surface;
 
     int x = patblt->nLeftRect;
     int y = patblt->nTopRect;
@@ -177,7 +177,8 @@ void guac_rdp_gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt) {
 
         /* If blackness, send black rectangle */
         case 0x00:
-            guac_common_surface_rect(current_surface, x, y, w, h, 0, 0, 0);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0x00, 0x00, 0x00, 0xFF);
             break;
 
         /* If NOP, do nothing */
@@ -187,15 +188,17 @@ void guac_rdp_gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt) {
         /* If operation is just a copy, send foreground only */
         case 0xCC:
         case 0xF0:
-            guac_common_surface_rect(current_surface, x, y, w, h,
+            guac_common_surface_set(current_surface, x, y, w, h,
                     (patblt->foreColor >> 16) & 0xFF,
                     (patblt->foreColor >> 8 ) & 0xFF,
-                    (patblt->foreColor      ) & 0xFF);
+                    (patblt->foreColor      ) & 0xFF,
+                    0xFF);
             break;
 
         /* If whiteness, send white rectangle */
         case 0xFF:
-            guac_common_surface_rect(current_surface, x, y, w, h, 0xFF, 0xFF, 0xFF);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0xFF, 0xFF, 0xFF, 0xFF);
             break;
 
         /* Otherwise, invert entire rect */
@@ -210,7 +213,7 @@ void guac_rdp_gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt) {
 void guac_rdp_gdi_scrblt(rdpContext* context, SCRBLT_ORDER* scrblt) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_common_surface* current_surface = ((rdp_guac_client_data*) client->data)->current_surface;
+    guac_common_surface* current_surface = ((guac_rdp_client*) client->data)->current_surface;
     
     int x = scrblt->nLeftRect;
     int y = scrblt->nTopRect;
@@ -220,18 +223,18 @@ void guac_rdp_gdi_scrblt(rdpContext* context, SCRBLT_ORDER* scrblt) {
     int x_src = scrblt->nXSrc;
     int y_src = scrblt->nYSrc;
 
-    rdp_guac_client_data* data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     /* Copy screen rect to current surface */
-    guac_common_surface_copy(data->default_surface, x_src, y_src, w, h,
-                             current_surface, x, y);
+    guac_common_surface_copy(rdp_client->display->default_surface,
+            x_src, y_src, w, h, current_surface, x, y);
 
 }
 
 void guac_rdp_gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_common_surface* current_surface = ((rdp_guac_client_data*) client->data)->current_surface;
+    guac_common_surface* current_surface = ((guac_rdp_client*) client->data)->current_surface;
     guac_rdp_bitmap* bitmap = (guac_rdp_bitmap*) memblt->bitmap;
 
     int x = memblt->nLeftRect;
@@ -252,7 +255,8 @@ void guac_rdp_gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt) {
 
         /* If blackness, send black rectangle */
         case 0x00:
-            guac_common_surface_rect(current_surface, x, y, w, h, 0, 0, 0);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0x00, 0x00, 0x00, 0xFF);
             break;
 
         /* If NOP, do nothing */
@@ -263,11 +267,11 @@ void guac_rdp_gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt) {
         case 0xCC: 
 
             /* If not cached, cache if necessary */
-            if (bitmap->surface == NULL && bitmap->used >= 1)
+            if (bitmap->layer == NULL && bitmap->used >= 1)
                 guac_rdp_cache_bitmap(context, memblt->bitmap);
 
             /* If not cached, send as PNG */
-            if (bitmap->surface == NULL) {
+            if (bitmap->layer == NULL) {
                 if (memblt->bitmap->data != NULL) {
 
                     /* Create surface from image data */
@@ -286,8 +290,8 @@ void guac_rdp_gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt) {
 
             /* Otherwise, copy */
             else
-                guac_common_surface_copy(bitmap->surface, x_src, y_src, w, h,
-                                         current_surface, x, y);
+                guac_common_surface_copy(bitmap->layer->surface,
+                        x_src, y_src, w, h, current_surface, x, y);
 
             /* Increment usage counter */
             ((guac_rdp_bitmap*) bitmap)->used++;
@@ -296,19 +300,21 @@ void guac_rdp_gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt) {
 
         /* If whiteness, send white rectangle */
         case 0xFF:
-            guac_common_surface_rect(current_surface, x, y, w, h, 0xFF, 0xFF, 0xFF);
+            guac_common_surface_set(current_surface, x, y, w, h,
+                    0xFF, 0xFF, 0xFF, 0xFF);
             break;
 
         /* Otherwise, use transfer */
         default:
 
             /* If not available as a surface, make available. */
-            if (bitmap->surface == NULL)
+            if (bitmap->layer == NULL)
                 guac_rdp_cache_bitmap(context, memblt->bitmap);
 
-            guac_common_surface_transfer(bitmap->surface, x_src, y_src, w, h,
-                                         guac_rdp_rop3_transfer_function(client, memblt->bRop),
-                                         current_surface, x, y);
+            guac_common_surface_transfer(bitmap->layer->surface,
+                    x_src, y_src, w, h,
+                    guac_rdp_rop3_transfer_function(client, memblt->bRop),
+                    current_surface, x, y);
 
             /* Increment usage counter */
             ((guac_rdp_bitmap*) bitmap)->used++;
@@ -324,23 +330,31 @@ void guac_rdp_gdi_opaquerect(rdpContext* context, OPAQUE_RECT_ORDER* opaque_rect
 
     UINT32 color = guac_rdp_convert_color(context, opaque_rect->color);
 
-    guac_common_surface* current_surface = ((rdp_guac_client_data*) client->data)->current_surface;
+    guac_common_surface* current_surface = ((guac_rdp_client*) client->data)->current_surface;
 
     int x = opaque_rect->nLeftRect;
     int y = opaque_rect->nTopRect;
     int w = opaque_rect->nWidth;
     int h = opaque_rect->nHeight;
 
-    guac_common_surface_rect(current_surface, x, y, w, h,
+    guac_common_surface_set(current_surface, x, y, w, h,
             (color >> 16) & 0xFF,
             (color >> 8 ) & 0xFF,
-            (color      ) & 0xFF);
+            (color      ) & 0xFF,
+            0xFF);
 
 }
 
 /**
  * Updates the palette within a FreeRDP CLRCONV object using the new palette
  * entries provided by an RDP palette update.
+ *
+ * @param clrconv
+ *     The FreeRDP CLRCONV object to update.
+ *
+ * @param palette
+ *     An RDP palette update message containing the palette to store within the
+ *     given CLRCONV object.
  */
 static void guac_rdp_update_clrconv(CLRCONV* clrconv,
         PALETTE_UPDATE* palette) {
@@ -358,6 +372,14 @@ static void guac_rdp_update_clrconv(CLRCONV* clrconv,
 /**
  * Updates a raw ARGB32 palette using the new palette entries provided by an
  * RDP palette update.
+ *
+ * @param guac_palette
+ *     An array of 256 ARGB32 colors, with each entry corresponding to an
+ *     entry in the color palette.
+ *
+ * @param palette
+ *     An RDP palette update message containing the palette to store within the
+ *     given array of ARGB32 colors.
  */
 static void guac_rdp_update_palette(UINT32* guac_palette,
         PALETTE_UPDATE* palette) {
@@ -393,16 +415,18 @@ void guac_rdp_gdi_palette_update(rdpContext* context, PALETTE_UPDATE* palette) {
 void guac_rdp_gdi_set_bounds(rdpContext* context, rdpBounds* bounds) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     /* If no bounds given, clear bounding rect */
     if (bounds == NULL)
-        guac_common_surface_reset_clip(data->default_surface);
+        guac_common_surface_reset_clip(rdp_client->display->default_surface);
 
     /* Otherwise, set bounding rectangle */
     else
-        guac_common_surface_clip(data->default_surface, bounds->left, bounds->top,
-                                 bounds->right - bounds->left + 1, bounds->bottom - bounds->top + 1);
+        guac_common_surface_clip(rdp_client->display->default_surface,
+                bounds->left, bounds->top,
+                bounds->right - bounds->left + 1,
+                bounds->bottom - bounds->top + 1);
 
 }
 
@@ -413,13 +437,13 @@ void guac_rdp_gdi_end_paint(rdpContext* context) {
 void guac_rdp_gdi_desktop_resize(rdpContext* context) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* data = (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
-    guac_common_surface_resize(data->default_surface,
+    guac_common_surface_resize(rdp_client->display->default_surface,
             guac_rdp_get_width(context->instance),
             guac_rdp_get_height(context->instance));
 
-    guac_common_surface_reset_clip(data->default_surface);
+    guac_common_surface_reset_clip(rdp_client->display->default_surface);
 
     guac_client_log(client, GUAC_LOG_DEBUG, "Server resized display to %ix%i",
             guac_rdp_get_width(context->instance),
