@@ -1,29 +1,27 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
 #include "client.h"
-#include "guac_surface.h"
+#include "common/surface.h"
+#include "rdp.h"
 #include "rdp_color.h"
 #include "rdp_glyph.h"
 #include "rdp_settings.h"
@@ -102,9 +100,9 @@ void guac_rdp_glyph_new(rdpContext* context, rdpGlyph* glyph) {
 void guac_rdp_glyph_draw(rdpContext* context, rdpGlyph* glyph, int x, int y) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* guac_client_data = (rdp_guac_client_data*) client->data;
-    guac_common_surface* current_surface = guac_client_data->current_surface;
-    uint32_t fgcolor = guac_client_data->glyph_color;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_common_surface* current_surface = rdp_client->current_surface;
+    uint32_t fgcolor = rdp_client->glyph_color;
 
     /* Paint with glyph as mask */
     guac_common_surface_paint(current_surface, x, y, ((guac_rdp_glyph*) glyph)->surface,
@@ -129,8 +127,8 @@ void guac_rdp_glyph_begindraw(rdpContext* context,
         int x, int y, int width, int height, UINT32 fgcolor, UINT32 bgcolor) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    rdp_guac_client_data* guac_client_data =
-        (rdp_guac_client_data*) client->data;
+    guac_rdp_client* rdp_client =
+        (guac_rdp_client*) client->data;
 
     /* Fill background with color if specified */
     if (width != 0 && height != 0) {
@@ -138,15 +136,17 @@ void guac_rdp_glyph_begindraw(rdpContext* context,
         /* Convert background color */
         bgcolor = guac_rdp_convert_color(context, bgcolor);
 
-        guac_common_surface_rect(guac_client_data->current_surface, x, y, width, height,
-                                 (bgcolor & 0xFF0000) >> 16,
-                                 (bgcolor & 0x00FF00) >> 8,
-                                  bgcolor & 0x0000FF);
+        guac_common_surface_set(rdp_client->current_surface,
+                x, y, width, height,
+                (bgcolor & 0xFF0000) >> 16,
+                (bgcolor & 0x00FF00) >> 8,
+                (bgcolor & 0x0000FF),
+                0xFF);
 
     }
 
     /* Convert foreground color */
-    guac_client_data->glyph_color = guac_rdp_convert_color(context, fgcolor);
+    rdp_client->glyph_color = guac_rdp_convert_color(context, fgcolor);
 
 }
 
