@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 
@@ -65,6 +62,22 @@ typedef struct guac_rdp_keysym_desc {
      */
     const int* clear_keysyms;
 
+    /**
+     * Bitwise OR of the flags of all lock keys (ie: Caps lock, Num lock, etc.)
+     * which must be active for this keysym to be properly typed. Legal flags
+     * are KBD_SYNC_SCROLL_LOCK, KBD_SYNC_NUM_LOCK, KBD_SYNC_CAPS_LOCK, and
+     * KBD_SYNC_KANA_LOCK.
+     */
+    int set_locks;
+
+    /**
+     * Bitwise OR of the flags of all lock keys (ie: Caps lock, Num lock, etc.)
+     * which must be inactive for this keysym to be properly typed. Legal flags
+     * are KBD_SYNC_SCROLL_LOCK, KBD_SYNC_NUM_LOCK, KBD_SYNC_CAPS_LOCK, and
+     * KBD_SYNC_KANA_LOCK.
+     */
+    int clear_locks;
+
 } guac_rdp_keysym_desc;
 
 /**
@@ -99,34 +112,6 @@ struct guac_rdp_keymap {
 
 };
 
-/**
- * Static mapping from keysyms to scancodes.
- */
-typedef guac_rdp_keysym_desc guac_rdp_static_keymap[0x200][0x100];
-
-/**
- * Mapping from keysym to current state
- */
-typedef int guac_rdp_keysym_state_map[0x200][0x100];
-
-/**
- * Simple macro for determing whether a keysym can be stored (or retrieved)
- * from any keymap.
- */
-#define GUAC_RDP_KEYSYM_STORABLE(keysym) ((keysym) <= 0xFFFF || ((keysym) & 0xFFFF0000) == 0x01000000)
-
-/**
- * Simple macro for referencing the mapped value of a scancode for a given
- * keysym. The idea here is that a keysym of the form 0xABCD will map to
- * mapping[0xAB][0xCD] while a keysym of the form 0x100ABCD will map to
- * mapping[0x1AB][0xCD].
- */
-#define GUAC_RDP_KEYSYM_LOOKUP(keysym_mapping, keysym) (          \
-            (keysym_mapping)                                      \
-            [(((keysym) & 0xFF00) >> 8) | ((keysym) >> 16)]       \
-            [(keysym) & 0xFF]                                     \
-        )
-
 /**
  * The name of the default keymap, which MUST exist.
  */
@@ -196,6 +181,12 @@ extern const guac_rdp_keymap* GUAC_KEYMAPS[];
 
 /**
  * Return the keymap having the given name, if any, or NULL otherwise.
+ *
+ * @param name
+ *     The name of the keymap to find.
+ *
+ * @return
+ *     The keymap having the given name, or NULL if no such keymap exists.
  */
 const guac_rdp_keymap* guac_rdp_keymap_find(const char* name);
 
