@@ -1,50 +1,49 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
 #include "client.h"
+#include "common/cursor.h"
+#include "common/display.h"
+#include "rdp.h"
 #include "rdp_pointer.h"
 
 #include <cairo/cairo.h>
 #include <freerdp/freerdp.h>
 #include <guacamole/client.h>
-#include <guacamole/protocol.h>
-#include <guacamole/socket.h>
 
 #include <stdlib.h>
 
 void guac_rdp_pointer_new(rdpContext* context, rdpPointer* pointer) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_socket* socket = client->socket;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
+    /* Allocate buffer */
+    guac_common_display_layer* buffer = guac_common_display_alloc_buffer(
+            rdp_client->display, pointer->width, pointer->height);
 
     /* Allocate data for image */
     unsigned char* data =
         (unsigned char*) malloc(pointer->width * pointer->height * 4);
 
-    /* Allocate layer */
-    guac_layer* buffer = guac_client_alloc_buffer(client);
-
     cairo_surface_t* surface;
 
     /* Convert to alpha cursor if mask data present */
@@ -60,8 +59,7 @@ void guac_rdp_pointer_new(rdpContext* context, rdpPointer* pointer) {
         pointer->width, pointer->height, 4*pointer->width);
 
     /* Send surface to buffer */
-    guac_client_stream_png(client, socket, GUAC_COMP_SRC, buffer,
-            0, 0, surface);
+    guac_common_surface_draw(buffer->surface, 0, 0, surface);
 
     /* Free surface */
     cairo_surface_destroy(surface);
@@ -75,19 +73,23 @@ void guac_rdp_pointer_new(rdpContext* context, rdpPointer* pointer) {
 void guac_rdp_pointer_set(rdpContext* context, rdpPointer* pointer) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_socket* socket = client->socket;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     /* Set cursor */
-    guac_protocol_send_cursor(socket, pointer->xPos, pointer->yPos,
-            ((guac_rdp_pointer*) pointer)->layer,
-            0, 0, pointer->width, pointer->height);
+    guac_common_cursor_set_surface(rdp_client->display->cursor,
+            pointer->xPos, pointer->yPos,
+            ((guac_rdp_pointer*) pointer)->layer->surface);
 
 }
 
 void guac_rdp_pointer_free(rdpContext* context, rdpPointer* pointer) {
 
     guac_client* client = ((rdp_freerdp_context*) context)->client;
-    guac_client_free_buffer(client, ((guac_rdp_pointer*) pointer)->layer);
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_common_display_layer* buffer = ((guac_rdp_pointer*) pointer)->layer;
+
+    /* Free buffer */
+    guac_common_display_free_buffer(rdp_client->display, buffer);
 
 }
 
