@@ -1,29 +1,27 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 
 #include "config.h"
 #include "client.h"
-#include "guac_clipboard.h"
+#include "common/clipboard.h"
+#include "rdp.h"
 #include "rdp_fs.h"
 #include "rdp_svc.h"
 #include "rdp_stream.h"
@@ -82,19 +80,22 @@ static void __generate_upload_path(const char* filename, char* path) {
 
 }
 
-int guac_rdp_upload_file_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_upload_file_handler(guac_user* user, guac_stream* stream,
         char* mimetype, char* filename) {
 
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
     int file_id;
     guac_rdp_stream* rdp_stream;
     char file_path[GUAC_RDP_FS_MAX_PATH];
 
     /* Get filesystem, return error if no filesystem */
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO FS)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO FS)",
                 GUAC_PROTOCOL_STATUS_SERVER_ERROR);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -105,9 +106,9 @@ int guac_rdp_upload_file_handler(guac_client* client, guac_stream* stream,
     file_id = guac_rdp_fs_open(fs, file_path, ACCESS_GENERIC_WRITE, 0,
             DISP_FILE_OVERWRITE_IF, 0);
     if (file_id < 0) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (CANNOT OPEN)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (CANNOT OPEN)",
                 GUAC_PROTOCOL_STATUS_CLIENT_FORBIDDEN);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -120,31 +121,31 @@ int guac_rdp_upload_file_handler(guac_client* client, guac_stream* stream,
     stream->blob_handler = guac_rdp_upload_blob_handler;
     stream->end_handler = guac_rdp_upload_end_handler;
 
-    guac_protocol_send_ack(client->socket, stream, "OK (STREAM BEGIN)",
+    guac_protocol_send_ack(user->socket, stream, "OK (STREAM BEGIN)",
             GUAC_PROTOCOL_STATUS_SUCCESS);
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 
 }
 
-int guac_rdp_svc_pipe_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_svc_pipe_handler(guac_user* user, guac_stream* stream,
         char* mimetype, char* name) {
 
     guac_rdp_stream* rdp_stream;
-    guac_rdp_svc* svc = guac_rdp_get_svc(client, name);
+    guac_rdp_svc* svc = guac_rdp_get_svc(user->client, name);
 
     /* Fail if no such SVC */
     if (svc == NULL) {
-        guac_client_log(client, GUAC_LOG_ERROR,
+        guac_user_log(user, GUAC_LOG_ERROR,
                 "Requested non-existent pipe: \"%s\".",
                 name);
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO SUCH PIPE)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO SUCH PIPE)",
                 GUAC_PROTOCOL_STATUS_CLIENT_BAD_REQUEST);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
     else
-        guac_client_log(client, GUAC_LOG_ERROR,
+        guac_user_log(user, GUAC_LOG_ERROR,
                 "Inbound half of channel \"%s\" connected.",
                 name);
 
@@ -153,16 +154,16 @@ int guac_rdp_svc_pipe_handler(guac_client* client, guac_stream* stream,
     stream->blob_handler = guac_rdp_svc_blob_handler;
     rdp_stream->type = GUAC_RDP_INBOUND_SVC_STREAM;
     rdp_stream->svc = svc;
-    svc->input_pipe = stream;
 
     return 0;
 
 }
 
-int guac_rdp_clipboard_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_clipboard_handler(guac_user* user, guac_stream* stream,
         char* mimetype) {
 
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
     guac_rdp_stream* rdp_stream;
 
     /* Init stream data */
@@ -171,23 +172,25 @@ int guac_rdp_clipboard_handler(guac_client* client, guac_stream* stream,
     stream->end_handler = guac_rdp_clipboard_end_handler;
     rdp_stream->type = GUAC_RDP_INBOUND_CLIPBOARD_STREAM;
 
-    guac_common_clipboard_reset(client_data->clipboard, mimetype);
+    guac_common_clipboard_reset(rdp_client->clipboard, mimetype);
     return 0;
 
 }
 
-int guac_rdp_upload_blob_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_upload_blob_handler(guac_user* user, guac_stream* stream,
         void* data, int length) {
 
     int bytes_written;
     guac_rdp_stream* rdp_stream = (guac_rdp_stream*) stream->data;
 
     /* Get filesystem, return error if no filesystem 0*/
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO FS)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO FS)",
                 GUAC_PROTOCOL_STATUS_SERVER_ERROR);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -202,10 +205,10 @@ int guac_rdp_upload_blob_handler(guac_client* client, guac_stream* stream,
 
         /* On error, abort */
         if (bytes_written < 0) {
-            guac_protocol_send_ack(client->socket, stream,
+            guac_protocol_send_ack(user->socket, stream,
                     "FAIL (BAD WRITE)",
                     GUAC_PROTOCOL_STATUS_CLIENT_FORBIDDEN);
-            guac_socket_flush(client->socket);
+            guac_socket_flush(user->socket);
             return 0;
         }
 
@@ -216,14 +219,14 @@ int guac_rdp_upload_blob_handler(guac_client* client, guac_stream* stream,
 
     }
 
-    guac_protocol_send_ack(client->socket, stream, "OK (DATA RECEIVED)",
+    guac_protocol_send_ack(user->socket, stream, "OK (DATA RECEIVED)",
             GUAC_PROTOCOL_STATUS_SUCCESS);
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 
 }
 
-int guac_rdp_svc_blob_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_svc_blob_handler(guac_user* user, guac_stream* stream,
         void* data, int length) {
 
     guac_rdp_stream* rdp_stream = (guac_rdp_stream*) stream->data;
@@ -231,32 +234,35 @@ int guac_rdp_svc_blob_handler(guac_client* client, guac_stream* stream,
     /* Write blob data to SVC directly */
     guac_rdp_svc_write(rdp_stream->svc, data, length);
 
-    guac_protocol_send_ack(client->socket, stream, "OK (DATA RECEIVED)",
+    guac_protocol_send_ack(user->socket, stream, "OK (DATA RECEIVED)",
             GUAC_PROTOCOL_STATUS_SUCCESS);
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 
 }
 
-int guac_rdp_clipboard_blob_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_clipboard_blob_handler(guac_user* user, guac_stream* stream,
         void* data, int length) {
 
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
-    guac_common_clipboard_append(client_data->clipboard, (char*) data, length);
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_common_clipboard_append(rdp_client->clipboard, (char*) data, length);
 
     return 0;
 }
 
-int guac_rdp_upload_end_handler(guac_client* client, guac_stream* stream) {
+int guac_rdp_upload_end_handler(guac_user* user, guac_stream* stream) {
 
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
     guac_rdp_stream* rdp_stream = (guac_rdp_stream*) stream->data;
 
     /* Get filesystem, return error if no filesystem */
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO FS)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO FS)",
                 GUAC_PROTOCOL_STATUS_SERVER_ERROR);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -264,51 +270,61 @@ int guac_rdp_upload_end_handler(guac_client* client, guac_stream* stream) {
     guac_rdp_fs_close(fs, rdp_stream->upload_status.file_id);
 
     /* Acknowledge stream end */
-    guac_protocol_send_ack(client->socket, stream, "OK (STREAM END)",
+    guac_protocol_send_ack(user->socket, stream, "OK (STREAM END)",
             GUAC_PROTOCOL_STATUS_SUCCESS);
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
 
     free(rdp_stream);
     return 0;
 
 }
 
-int guac_rdp_clipboard_end_handler(guac_client* client, guac_stream* stream) {
+int guac_rdp_clipboard_end_handler(guac_user* user, guac_stream* stream) {
 
-    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
-    rdpChannels* channels = client_data->rdp_inst->context->channels;
-
-    RDP_CB_FORMAT_LIST_EVENT* format_list =
-        (RDP_CB_FORMAT_LIST_EVENT*) freerdp_event_new(
-            CliprdrChannel_Class,
-            CliprdrChannel_FormatList,
-            NULL, NULL);
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
 
     /* Terminate clipboard data with NULL */
-    guac_common_clipboard_append(client_data->clipboard, "", 1);
+    guac_common_clipboard_append(rdp_client->clipboard, "", 1);
 
-    /* Notify server that text data is now available */
-    format_list->formats = (UINT32*) malloc(sizeof(UINT32));
-    format_list->formats[0] = CB_FORMAT_TEXT;
-    format_list->formats[1] = CB_FORMAT_UNICODETEXT;
-    format_list->num_formats = 2;
+    /* Notify RDP server of new data, if connected */
+    freerdp* rdp_inst = rdp_client->rdp_inst;
+    if (rdp_inst != NULL) {
 
-    freerdp_channels_send_event(channels, (wMessage*) format_list);
+        rdpChannels* channels = rdp_inst->context->channels;
+
+        RDP_CB_FORMAT_LIST_EVENT* format_list =
+            (RDP_CB_FORMAT_LIST_EVENT*) freerdp_event_new(
+                CliprdrChannel_Class,
+                CliprdrChannel_FormatList,
+                NULL, NULL);
+
+        /* Notify server that text data is now available */
+        format_list->formats = (UINT32*) malloc(sizeof(UINT32) * 2);
+        format_list->formats[0] = CB_FORMAT_TEXT;
+        format_list->formats[1] = CB_FORMAT_UNICODETEXT;
+        format_list->num_formats = 2;
+
+        freerdp_channels_send_event(channels, (wMessage*) format_list);
+
+    }
 
     return 0;
 }
 
-int guac_rdp_download_ack_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_download_ack_handler(guac_user* user, guac_stream* stream,
         char* message, guac_protocol_status status) {
 
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
     guac_rdp_stream* rdp_stream = (guac_rdp_stream*) stream->data;
 
     /* Get filesystem, return error if no filesystem */
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO FS)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO FS)",
                 GUAC_PROTOCOL_STATUS_SERVER_ERROR);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -324,39 +340,39 @@ int guac_rdp_download_ack_handler(guac_client* client, guac_stream* stream,
         /* If bytes read, send as blob */
         if (bytes_read > 0) {
             rdp_stream->download_status.offset += bytes_read;
-            guac_protocol_send_blob(client->socket, stream,
+            guac_protocol_send_blob(user->socket, stream,
                     buffer, bytes_read);
         }
 
         /* If EOF, send end */
         else if (bytes_read == 0) {
-            guac_protocol_send_end(client->socket, stream);
-            guac_client_free_stream(client, stream);
+            guac_protocol_send_end(user->socket, stream);
+            guac_user_free_stream(user, stream);
             free(rdp_stream);
         }
 
         /* Otherwise, fail stream */
         else {
-            guac_client_log(client, GUAC_LOG_ERROR,
+            guac_user_log(user, GUAC_LOG_ERROR,
                     "Error reading file for download");
-            guac_protocol_send_end(client->socket, stream);
-            guac_client_free_stream(client, stream);
+            guac_protocol_send_end(user->socket, stream);
+            guac_user_free_stream(user, stream);
             free(rdp_stream);
         }
 
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
 
     }
 
-    /* Otherwise, return stream to client */
+    /* Otherwise, return stream to user */
     else
-        guac_client_free_stream(client, stream);
+        guac_user_free_stream(user, stream);
 
     return 0;
 
 }
 
-int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
+int guac_rdp_ls_ack_handler(guac_user* user, guac_stream* stream,
         char* message, guac_protocol_status status) {
 
     int blob_written = 0;
@@ -368,7 +384,7 @@ int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
     if (status != GUAC_PROTOCOL_STATUS_SUCCESS) {
         guac_rdp_fs_close(rdp_stream->ls_status.fs,
                 rdp_stream->ls_status.file_id);
-        guac_client_free_stream(client, stream);
+        guac_user_free_stream(user, stream);
         free(rdp_stream);
         return 0;
     }
@@ -388,7 +404,7 @@ int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
         if (!guac_rdp_fs_append_filename(absolute_path,
                     rdp_stream->ls_status.directory_name, filename)) {
 
-            guac_client_log(client, GUAC_LOG_DEBUG,
+            guac_user_log(user, GUAC_LOG_DEBUG,
                     "Skipping filename \"%s\" - filename is invalid or "
                     "resulting path is too long", filename);
 
@@ -414,12 +430,12 @@ int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
         /* Determine mimetype */
         const char* mimetype;
         if (file->attributes & FILE_ATTRIBUTE_DIRECTORY)
-            mimetype = GUAC_CLIENT_STREAM_INDEX_MIMETYPE;
+            mimetype = GUAC_USER_STREAM_INDEX_MIMETYPE;
         else
             mimetype = "application/octet-stream";
 
         /* Write entry */
-        blob_written |= guac_common_json_write_property(client, stream,
+        blob_written |= guac_common_json_write_property(user, stream,
                 &rdp_stream->ls_status.json_state, absolute_path, mimetype);
 
         guac_rdp_fs_close(rdp_stream->ls_status.fs, file_id);
@@ -430,9 +446,9 @@ int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
     if (filename == NULL) {
 
         /* Complete JSON object */
-        guac_common_json_end_object(client, stream,
+        guac_common_json_end_object(user, stream,
                 &rdp_stream->ls_status.json_state);
-        guac_common_json_flush(client, stream,
+        guac_common_json_flush(user, stream,
                 &rdp_stream->ls_status.json_state);
 
         /* Clean up resources */
@@ -441,21 +457,24 @@ int guac_rdp_ls_ack_handler(guac_client* client, guac_stream* stream,
         free(rdp_stream);
 
         /* Signal of stream */
-        guac_protocol_send_end(client->socket, stream);
-        guac_client_free_stream(client, stream);
+        guac_protocol_send_end(user->socket, stream);
+        guac_user_free_stream(user, stream);
 
     }
 
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 
 }
 
-int guac_rdp_download_get_handler(guac_client* client, guac_object* object,
+int guac_rdp_download_get_handler(guac_user* user, guac_object* object,
         char* name) {
 
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
     /* Get filesystem, ignore request if no filesystem */
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL)
         return 0;
 
@@ -463,7 +482,7 @@ int guac_rdp_download_get_handler(guac_client* client, guac_object* object,
     int file_id = guac_rdp_fs_open(fs, name, ACCESS_GENERIC_READ, 0,
             DISP_FILE_OPEN, 0);
     if (file_id < 0) {
-        guac_client_log(client, GUAC_LOG_INFO, "Unable to read file \"%s\"",
+        guac_user_log(user, GUAC_LOG_INFO, "Unable to read file \"%s\"",
                 name);
         return 0;
     }
@@ -489,17 +508,17 @@ int guac_rdp_download_get_handler(guac_client* client, guac_object* object,
                 sizeof(rdp_stream->ls_status.directory_name) - 1);
 
         /* Allocate stream for body */
-        guac_stream* stream = guac_client_alloc_stream(client);
+        guac_stream* stream = guac_user_alloc_stream(user);
         stream->ack_handler = guac_rdp_ls_ack_handler;
         stream->data = rdp_stream;
 
         /* Init JSON object state */
-        guac_common_json_begin_object(client, stream,
+        guac_common_json_begin_object(user, stream,
                 &rdp_stream->ls_status.json_state);
 
         /* Associate new stream with get request */
-        guac_protocol_send_body(client->socket, object, stream,
-                GUAC_CLIENT_STREAM_INDEX_MIMETYPE, name);
+        guac_protocol_send_body(user->socket, object, stream,
+                GUAC_USER_STREAM_INDEX_MIMETYPE, name);
 
     }
 
@@ -513,29 +532,32 @@ int guac_rdp_download_get_handler(guac_client* client, guac_object* object,
         rdp_stream->download_status.offset = 0;
 
         /* Allocate stream for body */
-        guac_stream* stream = guac_client_alloc_stream(client);
+        guac_stream* stream = guac_user_alloc_stream(user);
         stream->data = rdp_stream;
         stream->ack_handler = guac_rdp_download_ack_handler;
 
         /* Associate new stream with get request */
-        guac_protocol_send_body(client->socket, object, stream,
+        guac_protocol_send_body(user->socket, object, stream,
                 "application/octet-stream", name);
 
     }
 
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 }
 
-int guac_rdp_upload_put_handler(guac_client* client, guac_object* object,
+int guac_rdp_upload_put_handler(guac_user* user, guac_object* object,
         guac_stream* stream, char* mimetype, char* name) {
 
+    guac_client* client = user->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
     /* Get filesystem, return error if no filesystem */
-    guac_rdp_fs* fs = ((rdp_guac_client_data*) client->data)->filesystem;
+    guac_rdp_fs* fs = rdp_client->filesystem;
     if (fs == NULL) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (NO FS)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (NO FS)",
                 GUAC_PROTOCOL_STATUS_SERVER_ERROR);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -545,9 +567,9 @@ int guac_rdp_upload_put_handler(guac_client* client, guac_object* object,
 
     /* Abort on failure */
     if (file_id < 0) {
-        guac_protocol_send_ack(client->socket, stream, "FAIL (CANNOT OPEN)",
+        guac_protocol_send_ack(user->socket, stream, "FAIL (CANNOT OPEN)",
                 GUAC_PROTOCOL_STATUS_CLIENT_FORBIDDEN);
-        guac_socket_flush(client->socket);
+        guac_socket_flush(user->socket);
         return 0;
     }
 
@@ -563,9 +585,9 @@ int guac_rdp_upload_put_handler(guac_client* client, guac_object* object,
     stream->end_handler = guac_rdp_upload_end_handler;
 
     /* Acknowledge stream creation */
-    guac_protocol_send_ack(client->socket, stream, "OK (STREAM BEGIN)",
+    guac_protocol_send_ack(user->socket, stream, "OK (STREAM BEGIN)",
             GUAC_PROTOCOL_STATUS_SUCCESS);
-    guac_socket_flush(client->socket);
+    guac_socket_flush(user->socket);
     return 0;
 }
 
