@@ -1,31 +1,29 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
-#include "common.h"
-#include "display.h"
-#include "guac_surface.h"
-#include "types.h"
+#include "common/surface.h"
+#include "terminal/common.h"
+#include "terminal/display.h"
+#include "terminal/palette.h"
+#include "terminal/types.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -39,30 +37,6 @@
 #include <guacamole/socket.h>
 #include <pango/pangocairo.h>
 
-const guac_terminal_color guac_terminal_palette[16] = {
-
-    /* Normal colors */
-    {0x00, 0x00, 0x00}, /* Black   */
-    {0x99, 0x3E, 0x3E}, /* Red     */
-    {0x3E, 0x99, 0x3E}, /* Green   */
-    {0x99, 0x99, 0x3E}, /* Brown   */
-    {0x3E, 0x3E, 0x99}, /* Blue    */
-    {0x99, 0x3E, 0x99}, /* Magenta */
-    {0x3E, 0x99, 0x99}, /* Cyan    */
-    {0x99, 0x99, 0x99}, /* White   */
-
-    /* Intense colors */
-    {0x3E, 0x3E, 0x3E}, /* Black   */
-    {0xFF, 0x67, 0x67}, /* Red     */
-    {0x67, 0xFF, 0x67}, /* Green   */
-    {0xFF, 0xFF, 0x67}, /* Brown   */
-    {0x67, 0x67, 0xFF}, /* Blue    */
-    {0xFF, 0x67, 0xFF}, /* Magenta */
-    {0x67, 0xFF, 0xFF}, /* Cyan    */
-    {0xFF, 0xFF, 0xFF}, /* White   */
-
-};
-
 /**
  * Clears the currently-selected region, removing the highlight.
  */
@@ -75,7 +49,7 @@ static void __guac_terminal_display_clear_select(guac_terminal_display* display)
     guac_protocol_send_cfill(socket, GUAC_COMP_SRC, select_layer,
             0x00, 0x00, 0x00, 0x00);
 
-    guac_protocol_send_sync(socket, display->client->last_sent_timestamp);
+    guac_client_end_frame(display->client);
     guac_socket_flush(socket);
 
     /* Text is no longer selected */
@@ -130,24 +104,36 @@ int __guac_terminal_hash_codepoint(int codepoint) {
 int __guac_terminal_set_colors(guac_terminal_display* display,
         guac_terminal_attributes* attributes) {
 
-    int background, foreground;
+    const guac_terminal_color* background;
+    const guac_terminal_color* foreground;
 
     /* Handle reverse video */
     if (attributes->reverse != attributes->cursor) {
-        background = attributes->foreground;
-        foreground = attributes->background;
+        background = &attributes->foreground;
+        foreground = &attributes->background;
     }
     else {
-        foreground = attributes->foreground;
-        background = attributes->background;
+        foreground = &attributes->foreground;
+        background = &attributes->background;
     }
 
     /* Handle bold */
-    if (attributes->bold && foreground <= 7)
-        foreground += 8;
+    if (attributes->bold && !attributes->half_bright
+            && foreground->palette_index >= GUAC_TERMINAL_FIRST_DARK
+            && foreground->palette_index <= GUAC_TERMINAL_LAST_DARK) {
+        foreground = &display->palette[foreground->palette_index
+            + GUAC_TERMINAL_INTENSE_OFFSET];
+    }
 
-    display->glyph_foreground = foreground;
-    display->glyph_background = background;
+    display->glyph_foreground = *foreground;
+    display->glyph_background = *background;
+
+    /* Modify color if half-bright (low intensity) */
+    if (attributes->half_bright && !attributes->bold) {
+        display->glyph_foreground.red   /= 2;
+        display->glyph_foreground.green /= 2;
+        display->glyph_foreground.blue  /= 2;
+    }
 
     return 0;
 
@@ -166,12 +152,10 @@ int __guac_terminal_set(guac_terminal_display* display, int row, int col, int co
     char utf8[4];
 
     /* Use foreground color */
-    const guac_terminal_color* color =
-        &guac_terminal_palette[display->glyph_foreground];
+    const guac_terminal_color* color = &display->glyph_foreground;
 
     /* Use background color */
-    const guac_terminal_color* background =
-        &guac_terminal_palette[display->glyph_background];
+    const guac_terminal_color* background = &display->glyph_background;
 
     cairo_surface_t* surface;
     cairo_t* cairo;
@@ -262,7 +246,8 @@ int __guac_terminal_set(guac_terminal_display* display, int row, int col, int co
 
 guac_terminal_display* guac_terminal_display_alloc(guac_client* client,
         const char* font_name, int font_size, int dpi,
-        int foreground, int background) {
+        guac_terminal_color* foreground, guac_terminal_color* background,
+        const guac_terminal_color (*palette)[256]) {
 
     PangoFontMap* font_map;
     PangoFont* font;
@@ -296,6 +281,7 @@ guac_terminal_display* guac_terminal_display_alloc(guac_client* client,
     font = pango_font_map_load_font(font_map, context, display->font_desc);
     if (font == NULL) {
         guac_client_abort(display->client, GUAC_PROTOCOL_STATUS_SERVER_ERROR, "Unable to get font \"%s\"", font_name);
+        free(display);
         return NULL;
     }
 
@@ -303,11 +289,13 @@ guac_terminal_display* guac_terminal_display_alloc(guac_client* client,
     if (metrics == NULL) {
         guac_client_abort(display->client, GUAC_PROTOCOL_STATUS_SERVER_ERROR,
                 "Unable to get font metrics for font \"%s\"", font_name);
+        free(display);
         return NULL;
     }
 
-    display->default_foreground = display->glyph_foreground = foreground;
-    display->default_background = display->glyph_background = background;
+    display->default_foreground = display->glyph_foreground = *foreground;
+    display->default_background = display->glyph_background = *background;
+    display->default_palette = palette;
 
     /* Calculate character dimensions */
     display->char_width =
@@ -331,6 +319,9 @@ guac_terminal_display* guac_terminal_display_alloc(guac_client* client,
 
 void guac_terminal_display_free(guac_terminal_display* display) {
 
+    /* Free default palette. */
+    free((void*) display->default_palette);
+
     /* Free operations buffers */
     free(display->operations);
 
@@ -339,6 +330,50 @@ void guac_terminal_display_free(guac_terminal_display* display) {
 
 }
 
+void guac_terminal_display_reset_palette(guac_terminal_display* display) {
+
+    /* Reinitialize palette with default values */
+    if (display->default_palette) {
+        memcpy(display->palette, *display->default_palette,
+               sizeof(GUAC_TERMINAL_INITIAL_PALETTE));
+        return;
+    }
+
+    memcpy(display->palette, GUAC_TERMINAL_INITIAL_PALETTE,
+            sizeof(GUAC_TERMINAL_INITIAL_PALETTE));
+
+}
+
+int guac_terminal_display_assign_color(guac_terminal_display* display,
+        int index, const guac_terminal_color* color) {
+
+    /* Assignment fails if out-of-bounds */
+    if (index < 0 || index > 255)
+        return 1;
+
+    /* Copy color components */
+    display->palette[index].red   = color->red;
+    display->palette[index].green = color->green;
+    display->palette[index].blue  = color->blue;
+
+    /* Color successfully stored */
+    return 0;
+
+}
+
+int guac_terminal_display_lookup_color(guac_terminal_display* display,
+        int index, guac_terminal_color* color) {
+
+    /* Lookup fails if out-of-bounds */
+    if (index < 0 || index > 255)
+        return 1;
+
+    /* Copy color definition */
+    *color = display->palette[index];
+    return 0;
+
+}
+
 void guac_terminal_display_copy_columns(guac_terminal_display* display, int row,
         int start_column, int end_column, int offset) {
 
@@ -441,6 +476,10 @@ void guac_terminal_display_set_columns(guac_terminal_display* display, int row,
     int i;
     guac_terminal_operation* current;
 
+    /* Do nothing if glyph is empty */
+    if (character->width == 0)
+        return;
+
     /* Ignore operations outside display bounds */
     if (row < 0 || row >= display->height)
         return;
@@ -687,14 +726,11 @@ void __guac_terminal_display_flush_clear(guac_terminal_display* display) {
                 int rect_width, rect_height;
 
                 /* Color of the rectangle to draw */
-                int color;
+                const guac_terminal_color* color;
                 if (current->character.attributes.reverse != current->character.attributes.cursor)
-                   color = current->character.attributes.foreground;
+                   color = &current->character.attributes.foreground;
                 else
-                   color = current->character.attributes.background;
-
-                const guac_terminal_color* guac_color =
-                    &guac_terminal_palette[color];
+                   color = &current->character.attributes.background;
 
                 /* Current row within a subrect */
                 guac_terminal_operation* rect_current_row;
@@ -708,16 +744,16 @@ void __guac_terminal_display_flush_clear(guac_terminal_display* display) {
                     /* Find width */
                     for (rect_col=col; rect_col<display->width; rect_col++) {
 
-                        int joining_color;
+                        const guac_terminal_color* joining_color;
                         if (rect_current->character.attributes.reverse != rect_current->character.attributes.cursor)
-                           joining_color = rect_current->character.attributes.foreground;
+                           joining_color = &rect_current->character.attributes.foreground;
                         else
-                           joining_color = rect_current->character.attributes.background;
+                           joining_color = &rect_current->character.attributes.background;
 
                         /* If not identical operation, stop */
                         if (rect_current->type != GUAC_CHAR_SET
                                 || guac_terminal_has_glyph(rect_current->character.value)
-                                || joining_color != color)
+                                || guac_terminal_colorcmp(joining_color, color) != 0)
                             break;
 
                         /* Next column */
@@ -753,16 +789,16 @@ void __guac_terminal_display_flush_clear(guac_terminal_display* display) {
 
                     for (rect_col=0; rect_col<rect_width; rect_col++) {
 
-                        int joining_color;
+                        const guac_terminal_color* joining_color;
                         if (rect_current->character.attributes.reverse != rect_current->character.attributes.cursor)
-                           joining_color = rect_current->character.attributes.foreground;
+                           joining_color = &rect_current->character.attributes.foreground;
                         else
-                           joining_color = rect_current->character.attributes.background;
+                           joining_color = &rect_current->character.attributes.background;
 
                         /* Mark clear operations as NOP */
                         if (rect_current->type == GUAC_CHAR_SET
                                 && !guac_terminal_has_glyph(rect_current->character.value)
-                                && joining_color == color)
+                                && guac_terminal_colorcmp(joining_color, color) == 0)
                             rect_current->type = GUAC_CHAR_NOP;
 
                         /* Next column */
@@ -776,13 +812,14 @@ void __guac_terminal_display_flush_clear(guac_terminal_display* display) {
                 }
 
                 /* Send rect */
-                guac_common_surface_rect(
+                guac_common_surface_set(
                         display->display_surface,
                         col * display->char_width,
                         row * display->char_height,
                         rect_width * display->char_width,
                         rect_height * display->char_height,
-                        guac_color->red, guac_color->green, guac_color->blue);
+                        color->red, color->green, color->blue,
+                        0xFF);
 
             } /* end if clear operation */
 
@@ -845,6 +882,23 @@ void guac_terminal_display_flush(guac_terminal_display* display) {
 
 }
 
+void guac_terminal_display_dup(guac_terminal_display* display, guac_user* user,
+        guac_socket* socket) {
+
+    /* Create default surface */
+    guac_common_surface_dup(display->display_surface, user, socket);
+
+    /* Select layer is a child of the display layer */
+    guac_protocol_send_move(socket, display->select_layer,
+            display->display_layer, 0, 0, 0);
+
+    /* Send select layer size */
+    guac_protocol_send_size(socket, display->select_layer,
+            display->char_width  * display->width,
+            display->char_height * display->height);
+
+}
+
 void guac_terminal_display_commit_select(guac_terminal_display* display) {
     display->selection_committed = true;
 }
@@ -935,7 +989,7 @@ void guac_terminal_display_select(guac_terminal_display* display,
     guac_protocol_send_cfill(socket, GUAC_COMP_SRC, select_layer,
             0x00, 0x80, 0xFF, 0x60);
 
-    guac_protocol_send_sync(socket, display->client->last_sent_timestamp);
+    guac_client_end_frame(display->client);
     guac_socket_flush(socket);
 
 }
