@@ -1,33 +1,34 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 #include "config.h"
 
-#include "char_mappings.h"
-#include "terminal.h"
-#include "terminal_handlers.h"
-#include "types.h"
+#include "terminal/char_mappings.h"
+#include "terminal/palette.h"
+#include "terminal/terminal.h"
+#include "terminal/terminal_handlers.h"
+#include "terminal/types.h"
+#include "terminal/xparsecolor.h"
 
 #include <guacamole/client.h>
+#include <guacamole/protocol.h>
+#include <guacamole/socket.h>
 
 #include <stdbool.h>
 #include <stdlib.h>
@@ -48,6 +49,81 @@
  */
 #define GUAC_TERMINAL_OK          "\x1B[0n"
 
+/**
+ * Advances the cursor to the next row, scrolling if the cursor would otherwise
+ * leave the scrolling region. If the cursor is already outside the scrolling
+ * region, the cursor is prevented from leaving the terminal bounds.
+ *
+ * @param term
+ *     The guac_terminal whose cursor should be advanced to the next row.
+ */
+static void guac_terminal_linefeed(guac_terminal* term) {
+
+    /* Scroll up if necessary */
+    if (term->cursor_row == term->scroll_end)
+        guac_terminal_scroll_up(term, term->scroll_start,
+                term->scroll_end, 1);
+
+    /* Otherwise, just advance to next row if space remains */
+    else if (term->cursor_row < term->term_height - 1)
+        term->cursor_row++;
+
+}
+
+/**
+ * Moves the cursor backward to the previous row, scrolling if the cursor would
+ * otherwise leave the scrolling region. If the cursor is already outside the
+ * scrolling region, the cursor is prevented from leaving the terminal bounds.
+ *
+ * @param term
+ *     The guac_terminal whose cursor should be moved backward by one row.
+ */
+static void guac_terminal_reverse_linefeed(guac_terminal* term) {
+
+    /* Scroll down if necessary */
+    if (term->cursor_row == term->scroll_start)
+        guac_terminal_scroll_down(term, term->scroll_start,
+                term->scroll_end, 1);
+
+    /* Otherwise, move back one row if space remains */
+    else if (term->cursor_row > 0)
+        term->cursor_row--;
+
+}
+
+/**
+ * Sets the position of the cursor without exceeding terminal bounds. Values
+ * which are out of bounds will be shifted to the nearest legal boundary.
+ *
+ * @param term
+ *     The guac_terminal whose cursor position is being set.
+ *
+ * @param row
+ *     The desired new row position.
+ *
+ * @param col
+ *     The desired new column position.
+ */
+static void guac_terminal_move_cursor(guac_terminal* term, int row, int col) {
+
+    /* Ensure cursor row is within terminal bounds */
+    if (row >= term->term_height)
+        row = term->term_height - 1;
+    else if (row < 0)
+        row = 0;
+
+    /* Ensure cursor column is within terminal bounds */
+    if (col >= term->term_width)
+        col = term->term_width - 1;
+    else if (col < 0)
+        col = 0;
+
+    /* Update cursor position */
+    term->cursor_row = row;
+    term->cursor_col = col;
+
+}
+
 int guac_terminal_echo(guac_terminal* term, unsigned char c) {
 
     int width;
@@ -57,6 +133,12 @@ int guac_terminal_echo(guac_terminal* term, unsigned char c) {
 
     const int* char_mapping = term->char_mapping[term->active_char_set];
 
+    /* Echo to pipe stream if open and not starting an ESC sequence */
+    if (term->pipe_stream != NULL && c != 0x1B) {
+        guac_terminal_pipe_stream_write(term, c);
+        return 0;
+    }
+
     /* If using non-Unicode mapping, just map straight bytes */
     if (char_mapping != NULL) {
         codepoint = c;
@@ -116,30 +198,23 @@ int guac_terminal_echo(guac_terminal* term, unsigned char c) {
 
         /* Backspace */
         case 0x08:
-            if (term->cursor_col >= 1)
-                term->cursor_col--;
+            guac_terminal_move_cursor(term, term->cursor_row,
+                    term->cursor_col - 1);
             break;
 
         /* Tab */
         case 0x09:
-            term->cursor_col = guac_terminal_next_tab(term, term->cursor_col);
+            guac_terminal_move_cursor(term, term->cursor_row,
+                    guac_terminal_next_tab(term, term->cursor_col));
             break;
 
         /* Line feed / VT / FF */
         case '\n':
         case 0x0B: /* VT */
         case 0x0C: /* FF */
-            term->cursor_row++;
 
-            /* Scroll up if necessary */
-            if (term->cursor_row > term->scroll_end) {
-                term->cursor_row = term->scroll_end;
-
-                /* Scroll up by one row */        
-                guac_terminal_scroll_up(term, term->scroll_start,
-                        term->scroll_end, 1);
-
-            }
+            /* Advance to next row */
+            guac_terminal_linefeed(term);
 
             /* If automatic carriage return, fall through to CR handler */
             if (!term->automatic_carriage_return)
@@ -147,7 +222,7 @@ int guac_terminal_echo(guac_terminal* term, unsigned char c) {
 
         /* Carriage return */
         case '\r':
-            term->cursor_col = 0;
+            guac_terminal_move_cursor(term, term->cursor_row, 0);
             break;
 
         /* SO (activates character set G1) */
@@ -188,17 +263,7 @@ int guac_terminal_echo(guac_terminal* term, unsigned char c) {
             /* Wrap if necessary */
             if (term->cursor_col >= term->term_width) {
                 term->cursor_col = 0;
-                term->cursor_row++;
-            }
-
-            /* Scroll up if necessary */
-            if (term->cursor_row > term->scroll_end) {
-                term->cursor_row = term->scroll_end;
-
-                /* Scroll up by one row */        
-                guac_terminal_scroll_up(term, term->scroll_start,
-                        term->scroll_end, 1);
-
+                guac_terminal_linefeed(term);
             }
 
             /* If insert mode, shift other characters right by 1 */
@@ -259,50 +324,23 @@ int guac_terminal_escape(guac_terminal* term, unsigned char c) {
 
         /* Restore Cursor (DECRC) */
         case '8':
-
-            term->cursor_row = term->saved_cursor_row;
-            if (term->cursor_row >= term->term_height)
-                term->cursor_row = term->term_height - 1;
-
-            term->cursor_col = term->saved_cursor_col;
-            if (term->cursor_col >= term->term_width)
-                term->cursor_col = term->term_width - 1;
+            guac_terminal_move_cursor(term,
+                    term->saved_cursor_row,
+                    term->saved_cursor_col);
 
             term->char_handler = guac_terminal_echo; 
             break;
 
         /* Index (IND) */
         case 'D':
-            term->cursor_row++;
-
-            /* Scroll up if necessary */
-            if (term->cursor_row > term->scroll_end) {
-                term->cursor_row = term->scroll_end;
-
-                /* Scroll up by one row */        
-                guac_terminal_scroll_up(term, term->scroll_start,
-                        term->scroll_end, 1);
-
-            }
-
+            guac_terminal_linefeed(term);
             term->char_handler = guac_terminal_echo; 
             break;
 
         /* Next Line (NEL) */
         case 'E':
-            term->cursor_col = 0;
-            term->cursor_row++;
-
-            /* Scroll up if necessary */
-            if (term->cursor_row > term->scroll_end) {
-                term->cursor_row = term->scroll_end;
-
-                /* Scroll up by one row */        
-                guac_terminal_scroll_up(term, term->scroll_start,
-                        term->scroll_end, 1);
-
-            }
-
+            guac_terminal_move_cursor(term, term->cursor_row, 0);
+            guac_terminal_linefeed(term);
             term->char_handler = guac_terminal_echo; 
             break;
 
@@ -314,19 +352,7 @@ int guac_terminal_escape(guac_terminal* term, unsigned char c) {
 
         /* Reverse Linefeed */
         case 'M':
-
-            term->cursor_row--;
-
-            /* Scroll down if necessary */
-            if (term->cursor_row < term->scroll_start) {
-                term->cursor_row = term->scroll_start;
-
-                /* Scroll down by one row */        
-                guac_terminal_scroll_down(term, term->scroll_start,
-                        term->scroll_end, 1);
-
-            }
-
+            guac_terminal_reverse_linefeed(term);
             term->char_handler = guac_terminal_echo; 
             break;
 
@@ -341,8 +367,13 @@ int guac_terminal_escape(guac_terminal* term, unsigned char c) {
             guac_terminal_reset(term);
             break;
 
+        case '_':
+            term->char_handler = guac_terminal_apc;
+            break;
+
         default:
-            guac_client_log(term->client, GUAC_LOG_INFO, "Unhandled ESC sequence: %c", c);
+            guac_client_log(term->client, GUAC_LOG_DEBUG,
+                    "Unhandled ESC sequence: %c", c);
             term->char_handler = guac_terminal_echo; 
 
     }
@@ -410,6 +441,146 @@ static bool* __guac_terminal_get_flag(guac_terminal* term, int num, char private
 
 }
 
+/**
+ * Parses an xterm SGR sequence specifying the RGB values of a color.
+ *
+ * @param argc
+ *     The number of arguments within the argv array.
+ *
+ * @param argv
+ *     The SGR arguments to parse, with the first relevant argument the
+ *     red component of the RGB color.
+ *
+ * @param color
+ *     The guac_terminal_color structure which should receive the parsed
+ *     color values.
+ *
+ * @return
+ *     The number of arguments parsed, or zero if argv does not contain
+ *     enough elements to represent an RGB color.
+ */
+static int guac_terminal_parse_xterm256_rgb(int argc, const int* argv,
+        guac_terminal_color* color) {
+
+    /* RGB color palette entries require three arguments */
+    if (argc < 3)
+        return 0;
+
+    /* Read RGB components from arguments */
+    int red   = argv[0];
+    int green = argv[1];
+    int blue  = argv[2];
+
+    /* Ignore if components are out of range */
+    if (   red   < 0 || red   > 255
+        || green < 0 || green > 255
+        || blue  < 0 || blue  > 255)
+        return 3;
+
+    /* Store RGB components */
+    color->red   = (uint8_t) red;
+    color->green = (uint8_t) green;
+    color->blue  = (uint8_t) blue;
+
+    /* Color is not from the palette */
+    color->palette_index = -1;
+
+    /* Done */
+    return 3;
+
+}
+
+/**
+ * Parses an xterm SGR sequence specifying the index of a color within the
+ * 256-color palette.
+ *
+ * @param terminal
+ *     The terminal associated with the palette.
+ *
+ * @param argc
+ *     The number of arguments within the argv array.
+ *
+ * @param argv
+ *     The SGR arguments to parse, with the first relevant argument being
+ *     the index of the color.
+ *
+ * @param color
+ *     The guac_terminal_color structure which should receive the parsed
+ *     color values.
+ *
+ * @return
+ *     The number of arguments parsed, or zero if the palette index is
+ *     absent.
+ */
+static int guac_terminal_parse_xterm256_index(guac_terminal* terminal,
+        int argc, const int* argv, guac_terminal_color* color) {
+
+    /* 256-color palette entries require only one argument */
+    if (argc < 1)
+        return 0;
+
+    /* Copy palette entry */
+    guac_terminal_display_lookup_color(terminal->display, argv[0], color);
+
+    /* Done */
+    return 1;
+
+}
+
+/**
+ * Parses an xterm SGR sequence specifying the index of a color within the
+ * 256-color palette, or specfying the RGB values of a color. The number of
+ * arguments required by these sequences varies. If a 256-color sequence is
+ * recognized, the number of arguments parsed is returned.
+ *
+ * @param terminal
+ *     The terminal whose palette state should be used when parsing the xterm
+ *     256-color SGR sequence.
+ *
+ * @param argc
+ *     The number of arguments within the argv array.
+ *
+ * @param argv
+ *     The SGR arguments to parse, with the first relevant argument being
+ *     the first element of the array. In the case of an xterm 256-color
+ *     SGR sequence, the first element here will be either 2, for an
+ *     RGB color, or 5, for a 256-color palette index. All other values
+ *     are invalid and will not be parsed.
+ *
+ * @param color
+ *     The guac_terminal_color structure which should receive the parsed
+ *     color values.
+ *
+ * @return
+ *     The number of arguments parsed, or zero if argv does not point to
+ *     the first element of an xterm 256-color SGR sequence.
+ */
+static int guac_terminal_parse_xterm256(guac_terminal* terminal,
+        int argc, const int* argv, guac_terminal_color* color) {
+
+    /* All 256-color codes must have at least a type */
+    if (argc < 1)
+        return 0;
+
+    switch (argv[0]) {
+
+        /* RGB */
+        case 2:
+            return guac_terminal_parse_xterm256_rgb(
+                    argc - 1, &argv[1], color) + 1;
+
+        /* Palette index */
+        case 5:
+            return guac_terminal_parse_xterm256_index(terminal,
+                    argc - 1, &argv[1], color) + 1;
+
+    }
+
+    /* Invalid type */
+    return 0;
+
+}
+
 int guac_terminal_csi(guac_terminal* term, unsigned char c) {
 
     /* CSI function arguments */
@@ -479,9 +650,9 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 if (amount == 0) amount = 1;
 
                 /* Move cursor */
-                term->cursor_row -= amount;
-                if (term->cursor_row < 0)
-                    term->cursor_row = 0;
+                guac_terminal_move_cursor(term,
+                        term->cursor_row - amount,
+                        term->cursor_col);
 
                 break;
 
@@ -494,9 +665,9 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 if (amount == 0) amount = 1;
 
                 /* Move cursor */
-                term->cursor_row += amount;
-                if (term->cursor_row >= term->term_height)
-                    term->cursor_row = term->term_height - 1;
+                guac_terminal_move_cursor(term,
+                        term->cursor_row + amount,
+                        term->cursor_col);
 
                 break;
 
@@ -509,9 +680,9 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 if (amount == 0) amount = 1;
 
                 /* Move cursor */
-                term->cursor_col += amount;
-                if (term->cursor_col >= term->term_width)
-                    term->cursor_col = term->term_width - 1;
+                guac_terminal_move_cursor(term,
+                        term->cursor_row,
+                        term->cursor_col + amount);
 
                 break;
 
@@ -523,9 +694,9 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 if (amount == 0) amount = 1;
 
                 /* Move cursor */
-                term->cursor_col -= amount;
-                if (term->cursor_col < 0)
-                    term->cursor_col = 0;
+                guac_terminal_move_cursor(term,
+                        term->cursor_row,
+                        term->cursor_col - amount);
 
                 break;
 
@@ -536,13 +707,10 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 amount = argv[0];
                 if (amount == 0) amount = 1;
 
-                /* Move cursor */
-                term->cursor_row += amount;
-                if (term->cursor_row >= term->term_height)
-                    term->cursor_row = term->term_height - 1;
-
-                /* Reset to column 1 */
-                term->cursor_col = 0;
+                /* Move cursor down, reset to column 1 */
+                guac_terminal_move_cursor(term,
+                        term->cursor_row + amount,
+                        0);
 
                 break;
 
@@ -553,13 +721,10 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 amount = argv[0];
                 if (amount == 0) amount = 1;
 
-                /* Move cursor */
-                term->cursor_row -= amount;
-                if (term->cursor_row < 0)
-                    term->cursor_row = 0;
-
-                /* Reset to column 1 */
-                term->cursor_col = 0;
+                /* Move cursor up , reset to column 1 */
+                guac_terminal_move_cursor(term,
+                        term->cursor_row - amount,
+                        0);
 
                 break;
 
@@ -567,7 +732,7 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
             case '`':
             case 'G':
                 col = argv[0]; if (col != 0) col--;
-                term->cursor_col = col;
+                guac_terminal_move_cursor(term, term->cursor_row, col);
                 break;
 
             /* H: Move cursor */
@@ -577,8 +742,7 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                 row = argv[0]; if (row != 0) row--;
                 col = argv[1]; if (col != 0) col--;
 
-                term->cursor_row = row;
-                term->cursor_col = col;
+                guac_terminal_move_cursor(term, row, col);
                 break;
 
             /* J: Erase display */
@@ -689,7 +853,7 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
             /* d: Move cursor, current col */
             case 'd':
                 row = argv[0]; if (row != 0) row--;
-                term->cursor_row = row;
+                guac_terminal_move_cursor(term, row, term->cursor_col);
                 break;
 
             /* g: Clear tab */
@@ -740,6 +904,10 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                     else if (value == 1)
                         term->current_attributes.bold = true;
 
+                    /* Faint (low intensity) */
+                    else if (value == 2)
+                        term->current_attributes.half_bright = true;
+
                     /* Underscore on */
                     else if (value == 4)
                         term->current_attributes.underscore = true;
@@ -749,8 +917,10 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
                         term->current_attributes.reverse = true;
 
                     /* Normal intensity (not bold) */
-                    else if (value == 21 || value == 22)
+                    else if (value == 21 || value == 22) {
                         term->current_attributes.bold = false;
+                        term->current_attributes.half_bright = false;
+                    }
 
                     /* Reset underscore */
                     else if (value == 24)
@@ -762,13 +932,32 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
 
                     /* Foreground */
                     else if (value >= 30 && value <= 37)
-                        term->current_attributes.foreground = value - 30;
+                        guac_terminal_display_lookup_color(term->display,
+                                value - 30,
+                                &term->current_attributes.foreground);
 
-                    /* Underscore on, default foreground */
+                    /* Underscore on, default foreground OR 256-color
+                     * foreground */
                     else if (value == 38) {
-                        term->current_attributes.underscore = true;
-                        term->current_attributes.foreground =
-                            term->default_char.attributes.foreground;
+
+                        /* Attempt to set foreground with 256-color entry */
+                        int xterm256_length =
+                            guac_terminal_parse_xterm256(term,
+                                    argc - i - 1, &argv[i + 1],
+                                    &term->current_attributes.foreground);
+
+                        /* If valid 256-color entry, foreground has been set */
+                        if (xterm256_length > 0)
+                            i += xterm256_length;
+
+                        /* Otherwise interpret as underscore and default
+                         * foreground  */
+                        else {
+                            term->current_attributes.underscore = true;
+                            term->current_attributes.foreground =
+                                term->default_char.attributes.foreground;
+                        }
+
                     }
 
                     /* Underscore off, default foreground */
@@ -780,13 +969,33 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
 
                     /* Background */
                     else if (value >= 40 && value <= 47)
-                        term->current_attributes.background = value - 40;
+                        guac_terminal_display_lookup_color(term->display,
+                                value - 40,
+                                &term->current_attributes.background);
+
+                    /* 256-color background */
+                    else if (value == 48)
+                        i += guac_terminal_parse_xterm256(term,
+                                argc - i - 1, &argv[i + 1],
+                                &term->current_attributes.background);
 
                     /* Reset background */
                     else if (value == 49)
                         term->current_attributes.background =
                             term->default_char.attributes.background;
 
+                    /* Intense foreground */
+                    else if (value >= 90 && value <= 97)
+                        guac_terminal_display_lookup_color(term->display,
+                                value - 90 + GUAC_TERMINAL_FIRST_INTENSE,
+                                &term->current_attributes.foreground);
+
+                    /* Intense background */
+                    else if (value >= 100 && value <= 107)
+                        guac_terminal_display_lookup_color(term->display,
+                                value - 100 + GUAC_TERMINAL_FIRST_INTENSE,
+                                &term->current_attributes.background);
+
                 }
 
                 break;
@@ -834,26 +1043,20 @@ int guac_terminal_csi(guac_terminal* term, unsigned char c) {
 
             /* Restore Cursor */
             case 'u':
-
-                term->cursor_row = term->saved_cursor_row;
-                if (term->cursor_row >= term->term_height)
-                    term->cursor_row = term->term_height - 1;
-
-                term->cursor_col = term->saved_cursor_col;
-                if (term->cursor_col >= term->term_width)
-                    term->cursor_col = term->term_width - 1;
-
+                guac_terminal_move_cursor(term,
+                        term->saved_cursor_row,
+                        term->saved_cursor_col);
                 break;
 
             /* Warn of unhandled codes */
             default:
                 if (c != ';') {
 
-                    guac_client_log(term->client, GUAC_LOG_INFO,
+                    guac_client_log(term->client, GUAC_LOG_DEBUG,
                             "Unhandled CSI sequence: %c", c);
 
                     for (i=0; i<argc; i++)
-                        guac_client_log(term->client, GUAC_LOG_INFO,
+                        guac_client_log(term->client, GUAC_LOG_DEBUG,
                                 " -> argv[%i] = %i", i, argv[i]);
 
                 }
@@ -899,7 +1102,7 @@ int guac_terminal_set_directory(guac_terminal* term, unsigned char c) {
             term->upload_path_handler(term->client, filename);
         else
             guac_client_log(term->client, GUAC_LOG_DEBUG,
-                    "Cannot set upload path. File is transfer not enabled.");
+                    "Cannot set upload path. File transfer is not enabled.");
         length = 0;
     }
 
@@ -924,7 +1127,7 @@ int guac_terminal_download(guac_terminal* term, unsigned char c) {
             term->file_download_handler(term->client, filename);
         else
             guac_client_log(term->client, GUAC_LOG_DEBUG,
-                    "Cannot send file. File is transfer not enabled.");
+                    "Cannot send file. File transfer is not enabled.");
         length = 0;
     }
 
@@ -936,6 +1139,163 @@ int guac_terminal_download(guac_terminal* term, unsigned char c) {
 
 }
 
+int guac_terminal_open_pipe_stream(guac_terminal* term, unsigned char c) {
+
+    static char stream_name[2048];
+    static int length = 0;
+
+    /* Open pipe on ECMA-48 ST (String Terminator) */
+    if (c == 0x9C || c == 0x5C || c == 0x07) {
+
+        /* End stream name string */
+        stream_name[length++] = '\0';
+        length = 0;
+
+        /* Open new pipe stream */
+        guac_terminal_pipe_stream_open(term, stream_name);
+
+        /* Return to echo mode */
+        term->char_handler = guac_terminal_echo;
+
+    }
+
+    /* Otherwise, store character within stream name */
+    else if (length < sizeof(stream_name)-1)
+        stream_name[length++] = c;
+
+    return 0;
+
+}
+
+int guac_terminal_close_pipe_stream(guac_terminal* term, unsigned char c) {
+
+    /* Handle closure on ECMA-48 ST (String Terminator) */
+    if (c == 0x9C || c == 0x5C || c == 0x07) {
+
+        /* Close any existing pipe */
+        guac_terminal_pipe_stream_close(term);
+
+        /* Return to echo mode */
+        term->char_handler = guac_terminal_echo;
+
+    }
+
+    /* Ignore all other characters */
+    return 0;
+
+}
+
+int guac_terminal_window_title(guac_terminal* term, unsigned char c) {
+
+    static int position = 0;
+    static char title[4096];
+
+    guac_socket* socket = term->client->socket;
+
+    /* Stop on ECMA-48 ST (String Terminator */
+    if (c == 0x9C || c == 0x5C || c == 0x07) {
+
+        /* Terminate and reset stored title */
+        title[position] = '\0';
+        position = 0;
+
+        /* Send title as connection name */
+        guac_protocol_send_name(socket, title);
+        guac_socket_flush(socket);
+
+        term->char_handler = guac_terminal_echo;
+
+    }
+
+    /* Store all other characters within title, space permitting */
+    else if (position < sizeof(title) - 1)
+        title[position++] = (char) c;
+
+    return 0;
+
+}
+
+int guac_terminal_xterm_palette(guac_terminal* term, unsigned char c) {
+
+    /**
+     * Whether we are currently reading the color spec. If false, we are
+     * currently reading the color index.
+     */
+    static bool read_color_spec = false;
+
+    /**
+     * The index of the palette entry being modified.
+     */
+    static int index = 0;
+
+    /**
+     * The color spec string, valid only if read_color_spec is true.
+     */
+    static char color_spec[256];
+
+    /**
+     * The current position within the color spec string, valid only if
+     * read_color_spec is true.
+     */
+    static int color_spec_pos = 0;
+
+    /* If not reading the color spec, parse the index */
+    if (!read_color_spec) {
+
+        /* If digit, append to index */
+        if (c >= '0' && c <= '9')
+            index = index * 10 + c - '0';
+
+        /* If end of parameter, switch to reading the color */
+        else if (c == ';') {
+            read_color_spec = true;
+            color_spec_pos = 0;
+        }
+
+    }
+
+    /* Once the index has been parsed, read the color spec */
+    else {
+
+        /* Modify palette once index/spec pair has been read */
+        if (c == ';' || c == 0x9C || c == 0x5C || c == 0x07) {
+
+            guac_terminal_color color;
+
+            /* Terminate color spec string */
+            color_spec[color_spec_pos] = '\0';
+
+            /* Modify palette if color spec is valid */
+            if (!guac_terminal_xparsecolor(color_spec, &color))
+                guac_terminal_display_assign_color(term->display,
+                        index, &color);
+            else
+                guac_client_log(term->client, GUAC_LOG_DEBUG,
+                        "Invalid XParseColor() color spec: \"%s\"",
+                        color_spec);
+
+            /* Resume parsing index */
+            read_color_spec = false;
+            index = 0;
+
+        }
+
+        /* Append characters to color spec as long as available space is not
+         * exceeded */
+        else if (color_spec_pos < 255) {
+            color_spec[color_spec_pos++] = c;
+        }
+
+    }
+
+    /* Stop on ECMA-48 ST (String Terminator */
+    if (c == 0x9C || c == 0x5C || c == 0x07)
+        term->char_handler = guac_terminal_echo;
+
+    return 0;
+
+}
+
 int guac_terminal_osc(guac_terminal* term, unsigned char c) {
 
     static int operation = 0;
@@ -955,6 +1315,22 @@ int guac_terminal_osc(guac_terminal* term, unsigned char c) {
         else if (operation == 482201)
             term->char_handler = guac_terminal_set_directory;
 
+        /* Open and redirect output to pipe stream OSC */
+        else if (operation == 482202)
+            term->char_handler = guac_terminal_open_pipe_stream;
+
+        /* Close pipe stream OSC */
+        else if (operation == 482203)
+            term->char_handler = guac_terminal_close_pipe_stream;
+
+        /* Set window title OSC */
+        else if (operation == 0 || operation == 2)
+            term->char_handler = guac_terminal_window_title;
+
+        /* xterm 256-color palette redefinition */
+        else if (operation == 4)
+            term->char_handler = guac_terminal_xterm_palette;
+
         /* Reset parameter for next OSC */
         operation = 0;
 
@@ -966,7 +1342,8 @@ int guac_terminal_osc(guac_terminal* term, unsigned char c) {
 
     /* Stop on unrecognized character */
     else {
-        guac_client_log(term->client, GUAC_LOG_INFO, "Unexpected character in OSC: 0x%X", c);
+        guac_client_log(term->client, GUAC_LOG_DEBUG,
+                "Unexpected character in OSC: 0x%X", c);
         term->char_handler = guac_terminal_echo;
     }
 
@@ -1000,3 +1377,19 @@ int guac_terminal_ctrl_func(guac_terminal* term, unsigned char c) {
 
 }
 
+int guac_terminal_apc(guac_terminal* term, unsigned char c) {
+
+    /* xterm does not implement APC functions and neither do we. Look for the
+     * "ESC \" (string terminator) sequence, while ignoring other chars. */
+    static bool escaping = false;
+
+    if (escaping) {
+        if (c == '\\')
+            term->char_handler = guac_terminal_echo;
+        escaping = false;
+    }
+
+    if (c == 0x1B)
+        escaping = true;
+    return 0;
+}
